Object.mixin = function( receiver, supplier ) {
  return Object.getOwnPropertyNames( supplier ).reduce(function( receiver, property ) {
    return Object.defineProperty(
      receiver, property, Object.getOwnPropertyDescriptor( supplier, property )
    );
  }, receiver );
};

function Initial( defaults, initializer ) {
  [ defaults, initializer || {} ].reduce( Object.mixin, this );
}

function Vector( initializer ) {
  Initial.call(
    this, Vector.DEFAULTS, initializer || {}
  );
}

Vector.DEFAULTS = {
  x: 0, y: 0, z: 0
};

Vector.prototype.magnitude = function() {
  var x, y, z;
  x = this.x;
  y = this.y;
  z = this.z;
  return Math.sqrt( x * x + y * y + z * z );
};

Vector.prototype.dot = function( vec ) {
  return this.x * vec.x + this.y * vec.y + this.z * vec.z;
};

Vector.prototype.angleBetweenA = function( vec ) {
  return Math.acos(
    this.dot(vec) / (this.magnitude() * vec.magnitude())
  ) / Math.PI * 180;
};


Vector.prototype.angleBetweenB = function( vec ) {
  var theta = this.dot(vec) / (this.magnitude() * vec.magnitude());

  theta = theta / Math.PI * 180;

  return theta;
};


var v1 = new Vector({ x: 2, y: 5, z: 1 });
var v2 = new Vector({ x: 10, y: 20, z: 1 });


console.log( v1.angleBetweenA(v2) );
console.log( v1.angleBetweenB(v2) );
