var Board = require("../lib/board.js"),
    events = require("events"),
    util = require("util"),
    es6 = require("es6-collections"),
    temporal = require("temporal"),
    WeakMap = es6.WeakMap;

// DHT instance private data
var priv = new WeakMap();


/**
 * DHT
 * @constructor
 *
 * @description DHT (Digitial Humidity & Temperature)
 *
 * @param {Object} opts Options: pin, freq, range
 */
function DHT( opts ) {
  if ( !(this instanceof DHT) ) {
    return new DHT( opts );
  }

  var data, counter;

  data = Array.apply(null, new Array(6)).map(function(v) { return 0; });
  counter = 0;


  // temporal.queue();

  opts = Board.options( opts );

  // Hardware instance properties
  this.board = Board.mount( opts );
  this.firmata = this.board.firmata;
  this.mode = this.firmata.MODES.OUTPUT;
  this.pin = opts.pin || 0;

  // Set the pin to INPUT mode
  this.firmata.pinMode( this.pin, this.mode );
  this.firmata.digitalWrite( this.pin, this.firmata.HIGH );


  // Servo instance properties
  this.freq = opts.freq || 25;
  this.range = opts.range || [ 0, 1023 ];
  this.sensitivity = opts.sensitivity === undefined ? 1 : opts.sensitivity;


  temporal.loop( 500, function() {
    this.firmata.digitalWrite( this.pin, this.firmata.LOW );
    temporal.queue([
      {
        delay: 0,
        task: function( task ) {
          this.firmata.digitalWrite( this.pin, this.firmata.HIGH );
        }.bind(this)
      },
      {
        delay: 10,
        task: function( task ) {
          this.firmata.pinMode( this.pin, this.firmata.MODES.INPUT );
        }.bind(this)
      },
      {
        delay: 10,
        task: function( task ) {
          this.firmata.digitalRead( this.pin, function( data ) {
            console.log( data );
            if ( data === 1 ) {
              // console.log( data );
              loop.call(this);
            }
          }.bind(this));
        }.bind(this)
      }
    ]);
  }.bind(this));


  function loop() {
    this.firmata.digitalRead( this.pin, function( data ) {
      console.log( data );
      // if ( data === 1 ) {
      //   loop.call(this);
      // }
    }.bind(this));
    setImmediate(loop);
  }


  /*
  Object.defineProperties( this, {
    normalized: {
      get: function() {
        return value === null ? null :
          Board.map( value, this.range[0], this.range[1], 0, 255 );
      }
    },
    constrained: {
      get: function() {
        return value === null ? null :
          Board.constrain( this.normalized, 0, 255 );
      }
    },
    scaled: {
      get: function() {
        var mapped, constrain, scale;

        scale = priv.get( this ).scale;

        if ( scale && value !== null ) {
          mapped = Board.map( value, this.range[0], this.range[1], scale[0], scale[1] );
          constrain = Board.constrain( mapped, scale[0], scale[1] );

          return constrain;
        }
        return this.constrained;
      }
    },
    value: {
      get: function() {
        var state;

        state = priv.get( this );

        if ( state.scale ) {
          return this.scaled;
        }

        return value;
      }
    }
  });
*/
}

util.inherits( DHT, events.EventEmitter );


/**
 * EXPERIMENTAL
 *
 * within When value is within the provided range, execute callback
 *
 * @param {Number} range Upperbound, converted into an array,
 *                       where 0 is lowerbound
 * @param {Function} callback Callback to execute when value falls inside range
 * @return {Object} instance
 *
 *
 * @param {Array} range Lower to Upper bounds [ low, high ]
 * @param {Function} callback Callback to execute when value falls inside range
 * @return {Object} instance
 *
 */
DHT.prototype.within = function( range, callback ) {
  var upper;

  if ( typeof range === "number" ) {
    upper = range;
    range = [ 0, upper ];
  }

  if ( !Array.isArray(range) ) {
    this.emit( "error", { message: "range must be an array" } );
    return;
  }

  // Use the continuous read event for high resolution
  this.on("read", function() {
    if ( this.value >= range[0] && this.value <= range[1] ) {
      callback.call( this, null );
    }
  }.bind(this));


  return this;
};

module.exports = DHT;

// Reference
// http://itp.nyu.edu/physcomp/Labs/Servo
// http://arduinobasics.blogspot.com/2011/05/arduino-uno-flex-sensor-and-leds.html



// TODO:
// Update comments/docs
